<?php

namespace Drupal\ld_json_schema\Form;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides backend module admin config page.
 *
 * @Block(
 *   id = "ld_json_schema_config_form",
 *   admin_label = @Translation("Ld JSON Schema Config Form"),
 *   category = @Translation("SEO"),
 * )
 *
 * Contains \Drupal\ld_json_schema\Form\SimpleConfigForm.
 */
class LdJsonSchemaConfigForm extends ConfigFormBase {

  /**
   * Config group settings.
   *
   * @var array
   */
  protected $settings;

  /**
   * Number of config elements.
   *
   * @var int
   */
  protected $number;

  /**
   * Fields type.
   *
   * @var array
   */
  protected $fieldGroups = [
    'elements',
  ];

  /**
   * Constructs a PdfTemplatesAdminForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    parent::__construct($config_factory);
    $config = $this->config('ld_json_schema.settings');

    foreach ($this->fieldGroups as $field) {
      $this->settings[$field] = $config->get('ld_json_schema.' . $field);
      $this->number[$field] = isset($this->settings[$field]) ? count($this->settings[$field]) : 1;
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'ld_json_schema.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ld_json_schema_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    foreach ($this->fieldGroups as $field) {
      $fieldClass = strtolower($field);

      $form['legend'] = [
        '#markup' => $this->t('You can add custom Ld JSON Schema to any page by URL. It may be useful for the custom routed pages that are not nodes or views.<br>
Schema will add new (not override existing) application/ld+json section.<br>
To add JSON Schema to nodes or views you can use the appropriate field that is provided by this module in the meta tags section.<br>
For views, you also may use the metatag_views module. It is optional because you can do this without it.<br>'),
      ];

      $form[$field] = [
        '#type' => 'details',
        '#title' => $this->t("@field", ['@field' => $field]),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
        '#open' => TRUE,
      ];

      $form[$field]['field_group'] = [
        '#type' => 'container',
        '#attributes' => ['id' => $fieldClass . '-field-wrapper'],
      ];

      $group_num = $this->number[$field];
      if ($group_num) {
        $settings = !empty($this->settings['elements']) ? $this->settings['elements'] : [];
        foreach ($settings as $i => $value) {
          $form[$field]['field_group']['field' . $i] = [
            '#type' => 'fieldset',
            '#title' => $this->t('Item #@num', ['@num' => $i]),
          ];

          $form[$field]['field_group']['field' . $i][$fieldClass . '-uri_' . $i] = [
            '#type' => 'textfield',
            '#title' => $this->t('URI'),
            '#description' => $this->t('URI should start with / symbol.'),
            '#required' => TRUE,
            '#default_value' => isset($value['uri']) ? $value['uri'] : NULL,
          ];

          $form[$field]['field_group']['field' . $i][$fieldClass . '-json_' . $i] = [
            '#type' => 'textarea',
            '#title' => $this->t('Raw JSON'),
            '#description' => $this->t('Raw JSON should have valid JSON format.'),
            '#required' => TRUE,
            '#default_value' => isset($value['json']) ? $value['json'] : NULL,
          ];

          $form[$field]['field_group']['field' . $i][$fieldClass . 'remove_group' . $i] = [
            '#type' => 'submit',
            '#value' => $this->t('Remove item #@number', ['@number' => $i]),
            '#name' => 'remove_field-' . $fieldClass,
            '#submit' => [$this, '::ajaxHandleRemoveGroup'],
            '#field' => $field,
            '#number' => $i,
            '#ajax' => [
              'callback' => [$this, 'ajaxHandleGroupCallback'],
              'wrapper' => $fieldClass . '-field-wrapper',
              'method' => 'replace',
            ],
            '#limit_validation_errors' => [['::ajaxHandleRemoveGroup']],
          ];
        }
      }

      $form[$field]['add_group'] = [
        '#type' => 'submit',
        '#value' => $this->t('Add new item'),
        '#name' => 'add_field-' . $fieldClass,
        '#submit' => [$this, '::ajaxHandleAddGroup'],
        '#field' => $field,
        '#ajax' => [
          'callback' => [$this, 'ajaxHandleGroupCallback'],
          'wrapper' => $fieldClass . '-field-wrapper',
          'method' => 'replace',
        ],
        '#limit_validation_errors' => [['::ajaxHandleAddGroup']],
      ];

    }

    return $form;
  }

  /**
   * Updated form part on ajax callback.
   */
  public function ajaxHandleGroupCallback($form, FormStateInterface $form_state) {
    $field = $form_state->getTriggeringElement()['#field'];
    return $form[$field]['field_group'];
  }

  /**
   * Add group submit function.
   */
  public function ajaxHandleAddGroup(array &$form, FormStateInterface $form_state) {
    $field = $form_state->getTriggeringElement()['#field'];
    $this->number[$field]++;
    $this->settings[$field][] = [];
    $form_state->setRebuild();
  }

  /**
   * Remove group submit function.
   */
  public function ajaxHandleRemoveGroup(array &$form, FormStateInterface $form_state) {
    $field = $form_state->getTriggeringElement()['#field'];
    $number = $form_state->getTriggeringElement()['#number'];
    $this->number[$field]--;
    unset($this->settings[$field][$number]);
    $form_state->setRebuild();
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    foreach ($this->fieldGroups as $field) {
      $fieldClass = strtolower($field);
      $fields = !empty($this->settings[$field]) ? $this->settings[$field] : [];
      foreach ($fields as $i => $value) {
        $url = $form_state->getValue($fieldClass . '-uri_' . $i);
        $json = $form_state->getValue($fieldClass . '-json_' . $i);

        if ($url[0] != "/") {
          $form_state->setErrorByName($fieldClass . '-uri_' . $i, \Drupal::translation()->translate('URI should start with / symbol.'));
        }

        if (empty($json) || !Json::decode($json)) {
          $form_state->setErrorByName($fieldClass . '-json_' . $i, \Drupal::translation()->translate('JSON field value should be a valid JSON.'));
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('ld_json_schema.settings');

    $settings = [];
    foreach ($this->fieldGroups as $field) {
      $fieldClass = strtolower($field);
      foreach ($this->settings[$field] as $i => $value) {
        $settings[$field][$i] = [
          'uri' => $form_state->getValue($fieldClass . '-uri_' . $i),
          'json' => $form_state->getValue($fieldClass . '-json_' . $i),
        ];
      }

      $config->set('ld_json_schema.' . $field, $settings[$field]);
    }
    $config->save();

    drupal_flush_all_caches();
    return parent::submitForm($form, $form_state);
  }

}
